{
This file is part of Finally.

Finally is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Finally is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Finally; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
.module       FINALLY;

.include <system.k>;
.include <midi.k>;

.global         handle_MIDI_controller;
.external       process_midi;

{ signal processing variables ------------------------ }


{ internal variables --------------------------------- }
.var last_input, last_output;
.init last_output:0;

.var input1,input2;
.var accumulator,tempacc;
.init accumulator:0;

{ parameters ----------------------------------------- }

.var opList[8];
.init opList:0,0,0,0, 0,0,0,0;

.var valList[8];
.init valList:0,0,0,0, 0,0,0,0;

{ codec init variables ------------------------------- }

.var/dm/ram/circ                rx_buf[3];      { Status + L data + R data }
.var/dm/ram/circ                tx_buf[3];      { Cmd + L data + R data    }
.var/dm/ram/circ                init_cmds[13];
.var/dm                         stat_flag;

.init tx_buf:   0xc000, 0x0000, 0x0000; { Initially set MCE        }

.init init_cmds:0xc007,0xc107,0xc288,0xc388,0xc488,0xc588,0xc680,0xc780,
                0xc85c,0xc909,0xca00,0xcc40,0xcd00;

{ Interrupt vector table }
        jump start;  rti; rti; rti;     {00: reset }
        rti;         rti; rti; rti;     {04: IRQ2 }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }
        ar = dm(stat_flag);             {10: SPORT0 tx }
        ar = pass ar;
        if eq rti;
        jump next_cmd;
        jump input_samples;             {14: SPORT1 rx }
                     rti; rti; rti;
        rti;         rti; rti; rti;     {18: IRQE }
        rti;         rti; rti; rti;     {1c: BDMA }
        jump irq1isr;rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
        rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }


{ ADSP 2181 intialization }

start:
        m7 = 0;
        l7 = 0;
        ax0 = b#0000100000000000;
        dm (System_Control_Reg) = ax0;          { shut down sport 0 }

        i7 = 0x3fe8;            { restore monitor timer handler }
        ar = pm (i7, m7);       { px implicit }
        i7 = 0x28;
        pm (i7, m7) = ar;

        i5 = ^rx_buf;l5 = %rx_buf;
        i6 = ^tx_buf;l6 = %tx_buf;
        i3 = ^init_cmds;l3 = %init_cmds;
        m1 = 1;m5 = 1;

{================== S E R I A L   P O R T   #0   S T U F F ==================}
        ax0 = b#0000110011010111;   dm (SPORT0_Autobuf) = ax0;
        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;

{============== S Y S T E M   A N D   M E M O R Y   S T U F F ==============}
        ax0 = b#0001100000000000;   dm (System_Control_Reg) = ax0;
        ifc = b#00000011111111;         { clear pending interrupt }
        nop;

        icntl = b#00010;
        mstat = b#1000000;

        { ADSP 1847 Codec intialization }
        ax0 = 1;
        dm(stat_flag) = ax0;            { clear flag }
        ena ints;                       { enable transmit interrupt }
        imask = b#0001000000;
        ax0 = dm (i6, m5);              { start interrupt }
        tx0 = ax0;

check_init:
        ax0 = dm (stat_flag);       { wait for entire init }
        af = pass ax0;              { buffer to be sent to }
        if ne jump check_init;      { the codec            }

        ay0 = 2;
check_aci1:
        ax0 = dm (rx_buf);          { once initialized, wait for codec }
        ar = ax0 and ay0;           { to come out of autocalibration }
        if eq jump check_aci1;      { wait for bit set }

check_aci2:
        ax0 = dm (rx_buf);          { wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci2;
        idle;

        ay0 = 0xbf3f;               { unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);   { unmute right DAC }
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;
        ifc = b#00000011111111;     { clear any pending interrupt }
        nop;

        l0=0; l1=0; l2=0; l3=0; l4=0; l7=0;
        imask = b#0000100101;       { enable rx0 interrupt }

        call set_baud_31250;
        jump init_params;

set_baud_31250:
.const CRYSTAL_FREQ_IN_kHZ=   16670;    {   Crystal speed related constant. }
.const  MIDIPERIOD = (CRYSTAL_FREQ_IN_kHZ * 2000 / (3 *  31250)) - 1;
        ax0=MIDIPERIOD;                 { autobaud not used }
        dm(TCOUNT)=ax0;
        dm(TPERIOD)=ax0;            { interrupts generated at 3x baudrate }
        rts;



init_params:
        nop;
        nop;
        nop;


{ command loop - check for incoming data on serial port }

main_loop:  
        ar = dm (CHAR_WAITING_FLAG);            { kbhit () }
        none = pass ar;
        if ne jump main_loop;
        call process_midi;              { has something }

        jump main_loop;


handle_MIDI_controller:
        ax1 = dm(midi_cont);
        ar  = dm(midi_val);

        ay1 =   1;none = ax1-ay1;if eq jump set_op1;
        ay1 =   2;none = ax1-ay1;if eq jump set_val1;
        ay1 =   3;none = ax1-ay1;if eq jump set_op2;
        ay1 =   4;none = ax1-ay1;if eq jump set_val2;

        ay1 =   5;none = ax1-ay1;if eq jump set_op3;
        ay1 =   6;none = ax1-ay1;if eq jump set_val3;
        ay1 =   7;none = ax1-ay1;if eq jump set_op4;
        ay1 =   8;none = ax1-ay1;if eq jump set_val4;

        ay1 =   9;none = ax1-ay1;if eq jump set_op5;
        ay1 =  10;none = ax1-ay1;if eq jump set_val5;
        ay1 =  11;none = ax1-ay1;if eq jump set_op6;
        ay1 =  12;none = ax1-ay1;if eq jump set_val6;

        ay1 =  13;none = ax1-ay1;if eq jump set_op7;
        ay1 =  14;none = ax1-ay1;if eq jump set_val7;
        ay1 =  15;none = ax1-ay1;if eq jump set_op8;
        ay1 =  16;none = ax1-ay1;if eq jump set_val8;

        ay1 = 127;none = ax1-ay1;if eq jump set_blah;
        rts;

set_op1:call fixup;dm(opList+0)=sr0;rts;
set_op2:call fixup;dm(opList+1)=sr0;rts;
set_op3:call fixup;dm(opList+2)=sr0;rts;
set_op4:call fixup;dm(opList+3)=sr0;rts;
set_op5:call fixup;dm(opList+4)=sr0;rts;
set_op6:call fixup;dm(opList+5)=sr0;rts;
set_op7:call fixup;dm(opList+6)=sr0;rts;
set_op8:call fixup;dm(opList+7)=sr0;rts;

set_val1:call fixup2;dm(valList+0)=sr0;rts;
set_val2:call fixup2;dm(valList+1)=sr0;rts;
set_val3:call fixup2;dm(valList+2)=sr0;rts;
set_val4:call fixup2;dm(valList+3)=sr0;rts;
set_val5:call fixup2;dm(valList+4)=sr0;rts;
set_val6:call fixup2;dm(valList+5)=sr0;rts;
set_val7:call fixup2;dm(valList+6)=sr0;rts;
set_val8:call fixup2;dm(valList+7)=sr0;rts;

{ fixup for ops...split into 8 diff operations }
fixup:  sr  = lshift ar by -4 (lo);
        rts;

{ fixup for values ... not used at this moment }
fixup2: sr0 = ar;
        rts;

set_blah:
        ay1 = dm(midi_val);
        rts;


{ SPORT0 interrupt handler  (called at 48KHz) }
input_samples:
        ena sec_reg;                { use shadow register bank }

        ax0 = dm(rx_buf +1);    { read a/d converters }
        ay0 = dm(rx_buf +2);
        dm(input1) = ax0;       { save em off for later use }
        dm(input2) = ay0;

        i0  = ^opList;
        i1  = ^valList;
        m0  = 0;
        m1  = 1;
        l0  = 0;
        l1  = 0;

        cntr = 8;
        ar  = dm(accumulator);


do singleOp until ce;

        ay1 = dm(i0,m1);        { get operation }
        ay0 = dm(i1,m1);        { get value }

        call specialValues;     { possibly modify ay0 - 126 = input1, 127 = input2 }

        call domeanop;          { leave result in ar }

singleOp: nop;



        dm(accumulator)=ar;     { save for next round }

        dm(tx_buf+ 1)=ar;       { let's listen to it... }
        dm(tx_buf+ 2)=ar;


thatwasfun:
        rti;    



{ operation # is in ay1 }
{ slider value still in ay0 }
{ input value is in ar }
{ output value will be in ar }
domeanop:
        ax1 = 0;none = ax1-ay1;if eq jump do_nothing;
        ax1 = 1;none = ax1-ay1;if eq jump do_and;
        ax1 = 2;none = ax1-ay1;if eq jump do_or;
        ax1 = 3;none = ax1-ay1;if eq jump do_xor;
        ax1 = 4;none = ax1-ay1;if eq jump do_shift;
        ax1 = 5;none = ax1-ay1;if eq jump do_add1;  { 8th }
        ax1 = 6;none = ax1-ay1;if eq jump do_mul;
        ax1 = 7;none = ax1-ay1;if eq jump do_add2;  { 8th }

do_nothing:
        rts;



{ for all of these operations, ---------------------------------------
  ay0 holds the parameter }

do_and:
        mr0 = ay0;
        sr  = lshift mr0 by 9 (lo);
        ay0 = sr0;
        dm(tempacc)=ar;
        ax0 = 0x01ff;   { 16 bits, we can only muck with 7 of them }
        ar  = ax0 or ay0;
        ay0 = ar;
        ar  = dm(tempacc);
        ar  = ar and ay0;
        rts;

do_or:
        mr0 = ay0;
        sr  = lshift mr0 by 8 (lo);
        ay0 = sr0;
        ar  = ar or ay0;
        rts;

do_xor:
        mr0 = ay0;
        sr  = lshift mr0 by 8 (lo);
        ay0 = sr0;
        ar  = ar xor ay0;
        rts;

do_shift:
        dm(tempacc)=ar;

        mr0 = ay0;
        sr  = lshift mr0 by -2 (lo);  { 7-bit -> 5-bit = 0-31 }
        ar  = sr0 - 16;                {-16<->15 }
        se  = ar;
        ar  = dm(tempacc);
        sr  =       ashift ar (lo);
        ax0 = sr0;
        ay0 = sr1;
        ar  = ax0 or ay0;
        rts;

{ fine add }
do_add1:
        ar  = ar -64;
        ar  = ar + ay0;
        rts;

do_mul:
        mx0 = ar;               { to be multiplied later }

        ar  = ay0;
        { get constant into proper range }
        sr  = lshift ar by 8 (lo);      { 007f -> 7f00 }
        ay0 = ar;
        ar  = sr0 or ay0;               { 7f00 | 007f = 7f7f }

        my0 = ar;                       { actually perform the mult. }
        mr  = mx0*my0(ss);              { mult result 0<->.9999 }

        ar  = mr1;
        rts;

        sr  = lshift mr1 by 2(lo);      { addl mult by four  0<-> 3.996 }
        ar  = sr0;

        rts;

{ coarse add }  { ignore the type }
do_add2:
        dm(tempacc) = ar;

        ar  = ay0;
        sr  = lshift ar by 5 (lo);      { multiply by 64 }
        ay0 = sr0;

        ar  = dm(tempacc);
        ar  = ar + ay0;

        rts;

{ possibly replace magic values with inputs! --------------------------- }
specialValues:
        ax1 = 126;none = ax1-ay0;if eq jump repInput1;
        ax1 = 127;none = ax1-ay0;if eq jump repInput2;
        rts;

{ replace slider value with audio input #1 }
repInput1:
        ay0 = dm(input1);
        rts;

repInput2:
        ay0 = dm(input2);
        rts;

{ transmit interrupt used for Codec initialization }
next_cmd:
        ena sec_reg;
        ax0 = dm (i3, m1);          { fetch next control word and }
        dm (tx_buf) = ax0;          { place in transmit slot 0    }
        ax0 = i3;
        ay0 = ^init_cmds;
        ar = ax0 - ay0;
        if gt rti;                  { rti if more control words still waiting }
        ax0 = 0xaf00;               { else set done flag and }
        dm (tx_buf) = ax0;          { remove MCE if done initialization }
        ax0 = 0;
        dm (stat_flag) = ax0;       { reset status flag }
        rti;

{ MIDI serial port ISR - actually just enables timer, which
  calls timer interrupt to read the serial port }
irq1isr:
        pop sts;
        ena timer;              { start timer now }
        rts;                    { note rts }

.endmod;
